import {
  Loader, Metadata, IEventParam, IDictionary,
} from '@zeedhi/core';

import {
  GridEditable, Select, Form,
} from '@zeedhi/common';

export class Dih52300MedicaoCaseiraController {
  public gridEditController = Loader.getInstance('GridEditableController');

  public crudController = Loader.getInstance('CrudController');

  public utilities = Loader.getInstance('Utilities');
  
  public fillCode({ component }: IEventParam<Select>) {
    if (component.value < 0) component.value = '0';
    component.value = component.value.padStart(5, '0');
  }

  public saveMedCas() {
    const form = Metadata.getInstance<Form>('formMedicaoCaseira');
    const grid: GridEditable = Metadata.getInstance('gridMedicaoCaseira');
    const editedRows = grid.getEditedRows() as IDictionary;
    
    if (form.validate() && editedRows.length > 0) {
      editedRows.forEach((row: IDictionary) => {
        row.CDMEDICASE = row.CDMEDICASE.padStart(5, '0');
      });
      
      this.crudController.saveRows(editedRows, 'medidaCaseira', grid).then(() => {
        this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      }).finally(() => {
        this.gridEditController.defaultState(grid);
      });
    }
  }
}
