import {
  Metadata, Loader, IEventParam, Http, IDictionary, dayjs as moment, I18n,
} from '@zeedhi/core';
import {
  Select, TreeGridEditable, Form, DialogService, Button, TextInput, Tabs,
} from '@zeedhi/common';

export class Pla01300DadosReceitaController {
  public crudController = Loader.getInstance('CrudController');

  private utilities = Loader.getInstance('Utilities');

  public gridEditController = Loader.getInstance('GridEditableController');

  public appController = Loader.getInstance('AppController');
  
  public setNameTab: string = '';
  
  public isNew = true;
  
  public qtdLiqBrut: string = '';
  
  public qtdPessoa: string = '';
  
  public getParamPlaneja() {
    const form = Metadata.getInstance<Form>('formUnidade');
    const cdfilial = form.value.UNIDADEFORM.CDFILIAL;
    
    this.utilities.getParamPlaneja(cdfilial).then((paramplaneja: IDictionary) => {
      this.qtdLiqBrut = paramplaneja?.IDQTLIQUBRUT;
    });
  }
  
  public setNameTabRecipe() {
    const tabPratos: Tabs = Metadata.getInstance('tabPratos');
    if (tabPratos.name === 'tabPratos') {
      if (tabPratos.activeTab === 2) this.setNameTab = 'subTabReceitaPadrao';
      else if (tabPratos.activeTab === 3) this.setNameTab = 'subTabReceitaTipoCard';
      else if (tabPratos.activeTab === 4) this.setNameTab = 'subTabReceitaLocal';
      else if (tabPratos.activeTab === 5) this.setNameTab = 'subTabReceitaServico';
      else if (tabPratos.activeTab === 6) this.setNameTab = 'subTabReceitaPadraoQualidade';
      else if (tabPratos.activeTab === 7) this.setNameTab = 'subTabReceitaRegiao';
    }
  }
  
  public gridProductSelected() {
    const tab = this.setNameTab;
    let grid: any;
    if (tab === 'subTabReceitaPadrao') grid = Metadata.getInstance('RECIPE_STANDARD');
    else if (tab === 'subTabReceitaLocal') grid = Metadata.getInstance('RECIPE_LOCAL');
    else if (tab === 'subTabReceitaServico') grid = Metadata.getInstance('RECIPE_SERVICE');
    else if (tab === 'subTabReceitaTipoCard') grid = Metadata.getInstance('RECIPE_TIPOCARD');
    return grid;
  }
  
  public setProductProdution({ component }: IEventParam<Select>) {
    const grid = this.gridProductSelected();
    
    if (grid) {
      const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
      const currentRow = plateTree.datasource.currentRow;
      const conditions = [
        {
          name: 'CDPRATO',
          operator: '=',
          value: currentRow.CDPRATO,
        },
        {
          name: 'CDPRODUTO',
          operator: '=',
          value: grid.datasource.currentRow.CDPRATO,
        },
        {
          name: 'CDFILAUXPRAT',
          operator: '=',
          value: currentRow.CDFILAUXPRAT,
        },
        {
          name: 'TCDPRODUTO',
          operator: '=',
          value: 'T',
        },
      ];
      
      component.datasource.filter = conditions;
      component.datasource.get();
      this.disableButtonsRecipeFalse();
    }
  }
  
  public validProductionProduct({ component }: IEventParam<Select>) {
    if (component.value !== null) {
      const grid = this.gridProductSelected();
      let receita: any;
      let formDadosRecipe: any;
      if (grid.name === 'RECIPE_STANDARD') {
        receita = 'P';
        formDadosRecipe = Metadata.getInstance<Form>('formDadosReceita');
      } else if (grid.name === 'RECIPE_LOCAL') {
        receita = 'F';
        formDadosRecipe = Metadata.getInstance<Form>('formDadosReceitaLocal');
      } else if (grid.name === 'RECIPE_SERVICE') {
        receita = 'S';
        formDadosRecipe = Metadata.getInstance<Form>('formDadosReceitaServico');
      }
      
      const rowDadosRecipe = formDadosRecipe.value;
      const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
      const currentRow = plateTree.datasource.currentRow;
      rowDadosRecipe.CDPRODUTO = component.value;
      rowDadosRecipe.CDPRATO = currentRow.CDPRATO;
      rowDadosRecipe.CDFILAUXPRAT = currentRow.CDFILAUXPRAT;
      rowDadosRecipe.RECEITA = receita;
      
      Http.post('/prato/validProductionProduct', { rowDadosRecipe }).then((callback) => {
        const result = callback.data.dataset.result;
        if (result.length > 0) {
          let receitas = '';
          result.forEach((item: IDictionary) => {
            if (item.TIPO === 'P') receitas += `${I18n.translate('pme::padrao')}: `;
            else if (item.TIPO === 'F') receitas += `${I18n.translate('pme::local')} (${item.CDFILIAL}): `;
            else if (item.TIPO === 'S') receitas += `${I18n.translate('pme::servico')} (${item.CDFILIAL}|${item.CDSERVFILI}): `;
            receitas += `${this.utilities.getValueWithMaskProduto(item.CDPRATO)} - ${item.NMPRATO}\n`;
          });
          this.utilities.showAlert('alertError', I18n.translate('pme::prodReceitaUtilizado', { PARAM1: receitas }), 'error');
          component.setValue('');
        }
      });
    }
  }
  
  public get visibleField() {
    const nameTab = this.setNameTab;
    if (nameTab === 'subTabReceitaRegiao' || nameTab === 'subTabReceitaPadraoQualidade') return false;
    return true;
  }
  
  public defaultStatus({ component }: IEventParam<Select>) {
    if (component.value === undefined || component.value === '' || component.value === null) component.value = 'S';
  }
  
  public defaultFinishing({ component }: IEventParam<Select>) {
    if (component.value === undefined || component.value === '' || component.value === null) component.setValue('M');
  }
  
  public defaultCookChill({ component }: IEventParam<Select>) {
    if (component.value === undefined || component.value === '' || component.value === null) component.setValue('N');
  }
  
  public validationCookChill({ component }: IEventParam<Select>) {
    const tab = this.setNameTab;
    let antecipacao;
    let armazenamento;
    if (tab === 'subTabReceitaPadrao') {
      const rowRefeicoesTransp = Metadata.getInstance<Form>('formRefeicoesTransp');
      antecipacao = Metadata.getInstance<TextInput>('QTLIMSEGPADR');
      armazenamento = rowRefeicoesTransp.getChildInstance<Select>('IDTPPRODCHILL');
    } else if (tab === 'subTabReceitaLocal') {
      const refTranspLocal = Metadata.getInstance<Form>('formRefeicoesTranspLocal');
      antecipacao = Metadata.getInstance<TextInput>('QTLIMSEGFILI');
      armazenamento = refTranspLocal.getChildInstance<Select>('IDTPPRODCHILL');
    } else if (tab === 'subTabReceitaServico') {
      const refTranspServico = Metadata.getInstance<Form>('formRefeicoesTranspServico');
      antecipacao = Metadata.getInstance<TextInput>('QTLIMSEGSERV');
      armazenamento = refTranspServico.getChildInstance<Select>('IDTPPRODCHILL');
    }
    // const armazenamento = Metadata.getInstance<Select>('IDTPPRODCHILL');
    
    if (antecipacao && armazenamento) {
      if (component.value === 'S') {
        antecipacao.disabled = false;
        antecipacao.value = '0';
        armazenamento.disabled = false;
        armazenamento.value = 'NENHUM';
      } else {
        antecipacao.disabled = true;
        antecipacao.value = null;
        antecipacao.removeValidation('required');
        armazenamento.disabled = true;
        armazenamento.value = null;
        armazenamento.removeValidation('required');
      }
    }
  }

  public calculateQtyPerson() {
    let qtAtiva = '';
    const grid = this.gridProductSelected();
    const QTDRECEITA = this.qtdLiqBrut;
    const prodReceita = grid.datasource.data;
    
    if (grid.name === 'RECIPE_STANDARD') {
      if (QTDRECEITA === 'B') qtAtiva = 'QTBRUTAPRUNR';
      else qtAtiva = 'QTPRODPRAT';
    } else if (grid.name === 'RECIPE_TIPOCARD') {
      if (QTDRECEITA === 'B') qtAtiva = 'QTBRUTATPPRTP';
      else qtAtiva = 'QTPRODPRATTP';
    } else if (grid.name === 'RECIPE_LOCAL') {
      if (QTDRECEITA === 'B') qtAtiva = 'QTBRUTALOUNR';
      else qtAtiva = 'QTPRODRECLOC';
    } else if (grid.name === 'RECIPE_SERVICE') {
      if (QTDRECEITA === 'B') qtAtiva = 'QTBRUTASEUNR';
      else qtAtiva = 'QTPRODRECSER';
    } else if (grid.name === 'RECIPE_PADRAOQUALIDADE') {
      if (QTDRECEITA === 'B') qtAtiva = 'QTBRUTAPADQUNR';
      else qtAtiva = 'QTPRODRECPADQ';
    } else if (grid.name === 'RECIPE_REGION') {
      if (QTDRECEITA === 'B') qtAtiva = 'QTBRUTAREGIAOUNR';
      else qtAtiva = 'QTPRODRECREGIAO';
    }
    
    this.returnFatConversao(prodReceita, qtAtiva);
  }
  
  public formDadosPrato() {
    const tab = this.setNameTab;
    let form: any;
    
    if (tab === 'subTabReceitaPadrao') form = Metadata.getInstance('formDadosReceita');
    else if (tab === 'subTabReceitaLocal') form = Metadata.getInstance('formDadosReceitaLocal');
    else if (tab === 'subTabReceitaServico') form = Metadata.getInstance('formDadosReceitaServico');
    else if (tab === 'subTabReceitaTipoCard') form = Metadata.getInstance('formDadosReceitaTipoCard');
    return form;
  }
  
  public returnFatConversao(prodReceita: any, qtAtiva: any) {
    const formDadosPlate = Metadata.getInstance<Form>('formDadosPrato');
    const sgUnidade = formDadosPlate.value.DSUNIDADEDP.SGUNIDADE;
    Http.post('/prato/returnFatConversao', { prodReceita, qtAtiva, sgUnidade }).then((callback) => {
      const qtdPessoa = callback.data.dataset.QTPESSOA;
      DialogService.show({
        name: 'pesoCalculadoPrato',
        type: 'warning',
        title: 'pme::atencao',
        text: I18n.translate('pme::pesoCalculadoPrato', { PARAM1: [qtdPessoa[0].toFixed(5)] }),
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.saveCalculo(qtdPessoa) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    });
  }
  
  public saveCalculo(qtdPessoa: any) {
    const form = this.formDadosPrato();
    if (form.name === 'formDadosReceita') form.value.QTPADRAOPRAT = qtdPessoa[0].toFixed(5);
    else if (form.name === 'formDadosReceitaLocal') form.value.QTPADRAOFILI = qtdPessoa[0].toFixed(5);
    else if (form.name === 'formDadosReceitaServico') form.value.QTPADRAOSERV = qtdPessoa[0].toFixed(5);
    else if (form.name === 'formDadosReceitaTipoCard') form.value.QTPADRAOPRATTP = qtdPessoa[0].toFixed(5);
    
    this.qtdPessoa = qtdPessoa[0].toFixed(5);
    this.salvaDadosRecipePlate();
    this.utilities.showAlert('alert', 'pme::calculadoComSucesso', 'success');
    DialogService.hide();
  }
  
  public salvaDadosRecipePlate() {
    const tab = this.setNameTab;
    
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRowTree = plateTree.datasource.currentRow;
    let row;
    
    if (tab === 'subTabReceitaPadrao') row = this.saveRecipePadrao(currentRowTree);
    else if (tab === 'subTabReceitaLocal') row = this.saveRecipeLocal(currentRowTree);
    else if (tab === 'subTabReceitaServico') row = this.saveRecipeService(currentRowTree);
    else if (tab === 'subTabReceitaTipoCard') row = this.saveRecipeTpCard(currentRowTree);
    
    if (row !== false) {
      if (this.validDateInactivation()) {
        Http.post('/prato/saveRecipePlate', { requestType: 'Row', row }).then(() => {
          this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
        }).catch((error) => {
          this.utilities.showAlert('alertError', error, 'error');
        }).finally(() => {
          this.setValuesRecipe();
          this.disableButtonsRecipeTrue();
        });
      }
    }
  }
  
  public saveRecipePadrao(currentRowTree: any) {
    const formDadosRecipe = Metadata.getInstance<Form>('formDadosReceita');
    const rowDadosRecipe = formDadosRecipe.value;
    const rowRefeicoesTransp = Metadata.getInstance<Form>('formRefeicoesTransp').value;
    const rowDadosTransporte = Metadata.getInstance<Form>('formDadosTransporte').value;
    
    if (formDadosRecipe.validate()) {
      const plate = currentRowTree.CDPRATO;
      const filAux = currentRowTree.CDFILAUXPRAT;
      
      rowDadosRecipe.VRPEMINPADR = rowDadosRecipe.VRPEMINPADR ? rowDadosRecipe.VRPEMINPADR : 0;
      
      const row = { ...rowDadosRecipe };
      row.QTPADRAOPRAT = this.qtdPessoa === '' ? rowDadosRecipe.QTPADRAOPRAT.toFixed(5) : this.qtdPessoa;
      row.CDPRATO = plate;
      row.CDFILAUXPRAT = filAux;
      row.TIPORECEITA = 'P';
      row.QTMEDICASEPRATPADR = rowDadosRecipe.QTMEDICASEPRATPADR ? rowDadosRecipe.QTMEDICASEPRATPADR : null;
      row.QTCONTAIPRAT = rowDadosTransporte.QTCONTAIPRAT ? rowDadosTransporte.QTCONTAIPRAT : '';
      row.VRPEMINPADR = rowDadosTransporte.VRPEMINPADR ? rowDadosTransporte.VRPEMINPADR : '';
      row.QTAMOSTRPADR = rowDadosTransporte.QTAMOSTRPADR ? rowDadosTransporte.QTAMOSTRPADR : '';
      row.IDFINALPADR = rowRefeicoesTransp.IDFINALPADR ? rowRefeicoesTransp.IDFINALPADR : 'M';
      row.CDPRODPRODUCAO = rowRefeicoesTransp.CDPRODPRODUCAO ? rowRefeicoesTransp.CDPRODPRODUCAO : '';
      row.NRCORPRIPRAT = this.hexToRGB(rowDadosRecipe.NRCORPRIPRAT);
      row.HRVALCONPRAT = rowDadosRecipe.HRVALCONPRAT ? this.calculateHour(rowDadosRecipe.HRVALCONPRAT) : '';
      row.DTINATIVAUTO = rowDadosRecipe.DTINATIVAUTO;
      row.__is_new = this.isNew;
      
      if (rowRefeicoesTransp.IDPRODCHILL === 'S') {
        if (rowRefeicoesTransp.QTLIMSEGPADR < 0) {
          this.utilities.showAlert('alert', 'pme::antecipacaoMaiorZero', 'warning');
          return false;
        } if (rowRefeicoesTransp.QTLIMSEGPADR === undefined) {
          this.utilities.showAlert('alert', 'pme::antecipacaoObrigatorio', 'warning');
          return false;
        } if (rowRefeicoesTransp.IDTPPRODCHILL === undefined) {
          this.utilities.showAlert('alert', 'pme::tipoArmazenamentoObrigatorio', 'warning');
          return false;
        }
      }
      
      if (row.CDMEDICASE !== null && row.QTMEDICASEPRATPADR === null) {
        this.utilities.showAlert('alert', 'pme::quantidadeObrigatorio', 'warning');
        return false;
      }
      
      if (row.QTMEDICASEPRATPADR !== null && row.QTMEDICASEPRATPADR < 0) {
        this.utilities.showAlert('alert', 'pme::qtdMaiorZero', 'warning');
        return false;
      }
      
      row.IDPRODCHILL = rowRefeicoesTransp.IDPRODCHILL ? rowRefeicoesTransp.IDPRODCHILL : 'N';
      if (rowRefeicoesTransp.IDPRODCHILL === 'N') {
        row.QTLIMSEGPADR = 0;
        row.IDTPPRODCHILL = 'NENHUM';
      } else {
        row.QTLIMSEGPADR = rowRefeicoesTransp.QTLIMSEGPADR ? rowRefeicoesTransp.QTLIMSEGPADR : '';
        row.IDTPPRODCHILL = rowRefeicoesTransp.IDTPPRODCHILL;
      }
      
      return row;
    } return false;
  }
  
  public saveRecipeLocal(currentRowTree: any) {
    const form = Metadata.getInstance<Form>('formDadosReceitaLocal');
    const formLocal = form.value;
    const refTranspLocal = Metadata.getInstance<Form>('formRefeicoesTranspLocal').value;
    const tranpLocal = Metadata.getInstance<Form>('formDadosTransporteLocal').value;
    const formFiltroLocal = Metadata.getInstance<Form>('FORM_LOCAL');
    const row = { ...formLocal };
    
    if (form.validate()) {
      row.CDFILIAL = formFiltroLocal.value.BRANCH_LOCAL.CDFILIAL;
      row.TIPORECEITA = 'L';
      row.CDPRATO = currentRowTree.CDPRATO;
      row.CDFILAUXPRAT = currentRowTree.CDFILAUXPRAT;
      
      row.CDPRODPRODUCAO = refTranspLocal.CDPRODPRODUCAO ? refTranspLocal.CDPRODPRODUCAO : '';
      
      row.QTPADRAOFILI = this.qtdPessoa === '' ? formLocal.QTPADRAOFILI.toFixed(5) : this.qtdPessoa;
      row.QTMEDICASEPRATFILI = formLocal.QTMEDICASEPRATFILI ? formLocal.QTMEDICASEPRATFILI : null;
      row.QTCONTAIFILI = tranpLocal.QTCONTAIFILI;
      row.VRPEMINFILI = tranpLocal.VRPEMINFILI;
      row.QTAMOSTRFILI = tranpLocal.QTAMOSTRFILI;
      row.NMFANTASIAPF = formLocal.NMFANTASIAPF;
      row.HRVALCONFILI = formLocal.HRVALCONFILI ? this.calculateHour(formLocal.HRVALCONFILI) : '';
      row.NRCORPRIFILI = this.hexToRGB(formLocal.NRCORPRIFILI);
      
      if (refTranspLocal.IDPRODCHILL === 'S') {
        if (refTranspLocal.QTLIMSEGFILI < 0) {
          this.utilities.showAlert('alert', 'pme::antecipacaoMaiorZero', 'warning');
          return false;
        } if (refTranspLocal.QTLIMSEGFILI === undefined) {
          this.utilities.showAlert('alert', 'pme::antecipacaoObrigatorio', 'warning');
          return false;
        } if (refTranspLocal.IDTPPRODCHILL === undefined) {
          this.utilities.showAlert('alert', 'pme::tipoArmazenamentoObrigatorio', 'warning');
          return false;
        }
      }
      
      if (row.CDMEDICASE !== null && row.QTMEDICASEPRATFILI === null) {
        this.utilities.showAlert('alert', 'pme::quantidadeObrigatorio', 'warning');
        return false;
      }
      
      if (row.CDMEDICASE !== null && row.QTMEDICASEPRATFILI < 0) {
        this.utilities.showAlert('alert', 'pme::qtdMaiorZero', 'warning');
        return false;
      }
      
      row.IDFINALFILI = refTranspLocal.IDFINALFILI ? refTranspLocal.IDFINALFILI : 'M';
      row.IDPRODCHILL = refTranspLocal.IDPRODCHILL ? refTranspLocal.IDPRODCHILL : 'N';
      if (refTranspLocal.IDPRODCHILL === 'N') {
        row.QTLIMSEGFILI = 0;
        row.IDTPPRODCHILL = 'NENHUM';
      } else {
        row.QTLIMSEGFILI = refTranspLocal.QTLIMSEGFILI ? refTranspLocal.QTLIMSEGFILI : '';
        row.IDTPPRODCHILL = refTranspLocal.IDTPPRODCHILL;
      }
      
      row.__is_new = this.isNew;
      return row;
    } return false;
  }
  
  public saveRecipeService(currentRowTree: any) {
    const form = Metadata.getInstance<Form>('formDadosReceitaServico');
    const formServico = form.value;
    const refTranspServico = Metadata.getInstance<Form>('formRefeicoesTranspServico').value;
    const tranpServico = Metadata.getInstance<Form>('formDadosTransporteServico').value;
    const formFiltroServ = Metadata.getInstance<Form>('FORM_SERVICE');
    
    if (form.validate()) {
      const row = { ...formServico };
      
      row.TIPORECEITA = 'S';
      row.CDFILIAL = formFiltroServ.value.BRANCH_SERVICE.CDFILIAL;
      row.CDSERVFILI = formFiltroServ.value.SERVICE.CDSERVFILI;
      row.CDPRATO = currentRowTree.CDPRATO;
      row.CDFILAUXPRAT = currentRowTree.CDFILAUXPRAT;
      
      row.CDMEDICASE = formServico.CDMEDICASE;
      
      row.CDPRODPRODUCAO = refTranspServico.CDPRODPRODUCAO ? refTranspServico.CDPRODPRODUCAO : '';
      
      row.QTPADRAOSERV = this.qtdPessoa === '' ? formServico.QTPADRAOSERV.toFixed(5) : this.qtdPessoa;
      row.NMFANTASIASE = formServico.NMFANTASIASE;
      row.QTMEDICASEPRATSERV = formServico.QTMEDICASEPRATSERV ? formServico.QTMEDICASEPRATSERV : null;
      row.QTCONTAISERV = tranpServico.QTCONTAISERV;
      row.VRPEMINSERV = tranpServico.VRPEMINSERV;
      row.QTAMOSTRSERV = tranpServico.QTAMOSTRSERV;
      
      if (refTranspServico.IDPRODCHILL === 'N') {
        row.QTLIMSEGSERV = 0;
        row.IDTPPRODCHILL = 'NENHUM';
      }

      if (refTranspServico.IDPRODCHILL === 'S') {
        if (refTranspServico.QTLIMSEGSERV < 0) {
          this.utilities.showAlert('alert', 'pme::antecipacaoMaiorZero', 'warning');
          return false;
        } if (refTranspServico.QTLIMSEGSERV === undefined) {
          this.utilities.showAlert('alert', 'pme::antecipacaoObrigatorio', 'warning');
          return false;
        } if (refTranspServico.IDTPPRODCHILL === undefined) {
          this.utilities.showAlert('alert', 'pme::tipoArmazenamentoObrigatorio', 'warning');
          return false;
        }
      }
      
      if (row.CDMEDICASE !== null && row.QTMEDICASEPRATSERV === null) {
        this.utilities.showAlert('alert', 'pme::quantidadeObrigatorio', 'warning');
        return false;
      }
      
      if (row.CDMEDICASE !== null && row.QTMEDICASEPRATSERV < 0) {
        this.utilities.showAlert('alert', 'pme::qtdMaiorZero', 'warning');
        return false;
      }
      
      row.IDFINALSERV = refTranspServico.IDFINALSERV ? refTranspServico.IDFINALSERV : 'M';
      row.IDPRODCHILL = refTranspServico.IDPRODCHILL ? refTranspServico.IDPRODCHILL : 'N';
      if (refTranspServico.IDPRODCHILL === 'N') {
        row.QTLIMSEGSERV = 0;
        row.IDTPPRODCHILL = 'NENHUM';
      } else {
        row.QTLIMSEGSERV = refTranspServico.QTLIMSEGSERV ? refTranspServico.QTLIMSEGSERV : '';
        row.IDTPPRODCHILL = refTranspServico.IDTPPRODCHILL;
      }
      
      row.HRVALCONSERV = formServico.HRVALCONSERV ? this.calculateHour(formServico.HRVALCONSERV) : '';
      row.NRCORPRISERV = this.hexToRGB(formServico.NRCORPRISERV);
      row.__is_new = this.isNew;
      return row;
    } return false;
  }
  
  public saveRecipeTpCard(currentRowTree: any) {
    const form = Metadata.getInstance<Form>('formDadosReceitaTipoCard');
    const formTipoCard = form.value;
    const tranpTipoCard = Metadata.getInstance<Form>('formDadosTransporteTipoCard').value;
    const formFiltroTpCard = Metadata.getInstance<Form>('FORM_TIPOCARD');
    const row = { ...formTipoCard };
    
    if (form.validate()) {
      row.TIPORECEITA = 'C';
      row.CDTIPOCARD = formFiltroTpCard.value.TIPOCARD.CDTIPOCARD;
      row.CDPRATO = currentRowTree.CDPRATO;
      row.CDFILAUXPRAT = currentRowTree.CDFILAUXPRAT;
      
      row.QTPADRAOPRATTP = this.qtdPessoa === '' ? formTipoCard.QTPADRAOPRATTP.tofixed(5) : this.qtdPessoa;
      
      row.NMFANTASIAPRTP = formTipoCard.NMFANTASIAPRTP;
      row.DSPRATO = formTipoCard.DSPRATO;
      row.QTCONTAIPRATTP = tranpTipoCard.QTCONTAIPRATTP;
      row.VRPEMINPADRTP = tranpTipoCard.VRPEMINPADRTP;
      row.QTAMOSTRPADRTP = tranpTipoCard.QTAMOSTRPADRTP;
      
      row.HRVALCONPRATTP = formTipoCard.HRVALCONPRATTP ? this.calculateHour(formTipoCard.HRVALCONPRATTP) : '';
      row.NRCORPRIPRATTP = this.hexToRGB(formTipoCard.NRCORPRIPRATTP);
      row.CDFILCENTPME = this.appController.getCdCentral();
      
      row.__is_new = this.isNew;
      return row;
    } return false;
  }
  
  public hexToRGB(corHex: any) {
    const corRgb = this.utilities.hexToRgbInline(corHex).split('.', 3);
    const rgb = this.utilities.RGBToTColor(corRgb[0], corRgb[1], corRgb[2]);
    return rgb;
  }
  
  public calculateHour(fieldHour: any) {
    const replacehHour = fieldHour.replace(':', '');
    const minutes = replacehHour.substr(-2);
    const hour = replacehHour.substr(0, 2);
    const total = (parseInt(hour, 10) + (minutes / 60));
    return total;
  }
  
  public setValuesRecipe() {
    this.setNameTabRecipe();
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRowTree = plateTree.datasource.currentRow;
    const tab = this.setNameTab;
    
    const plate = currentRowTree.CDPRATO;
    const filAux = currentRowTree.CDFILAUXPRAT;
    
    if (tab === 'subTabReceitaPadrao') {
      const form = Metadata.getInstance<Form>('formDadosReceita');
      const rowDadosRecipe = Metadata.getInstance<Form>('formDadosReceita').value;
      const rowRefeicoesTransp = Metadata.getInstance<Form>('formRefeicoesTransp').value;
      const rowDadosTransporte = Metadata.getInstance<Form>('formDadosTransporte').value;
      const medCase = form.getChildInstance<Select>('CDMEDICASE');
      const prodAcabado = Metadata.getInstance<Form>('formRefeicoesTransp').getChildInstance<Select>('CDPRODPRODUCAO');
      
      const filter: IDictionary = [];
      this.utilities.onMountedFilterFind(filter, 'CDPRATO', '=', plate);
      this.utilities.onMountedFilterFind(filter, 'CDFILAUXPRAT', '=', filAux);
      Http.post('/find/pratoPratpadrProdacabado', { requestType: 'FilterData', filter }).then((callback) => {
        const result = callback.data.data;
        if (result.length > 0) {
          this.isNew = false;
          const formatHour = result[0].HRVALCONPRAT ? this.formatHour(result[0].HRVALCONPRAT) : null;
          
          if (result[0].CDMEDICASE) {
            const medidaCaseira = {
              NMMEDICASE: result[0].NMMEDICASE,
              CDMEDICASE: result[0].CDMEDICASE,
            };
            medCase.datasource.data = [medidaCaseira];
            medCase.setValue(result[0].CDMEDICASE);
          } else rowDadosRecipe.CDMEDICASE = '';

          rowDadosRecipe.NMFANTASIAPR = result[0].NMFANTASIAPR;
          rowDadosRecipe.NRCORPRIPRAT = this.utilities.TColorToHex(result[0].NRCORPRIPRAT);
          rowDadosRecipe.HRVALCONPRAT = formatHour;
          rowDadosRecipe.QTPADRAOPRAT = result[0].QTPADRAOPRAT.toFixed(5);
          rowDadosRecipe.IDATIVO = result[0].IDATIVO;
          rowDadosRecipe.DTINATIVAUTO = result[0].DTINATIVAUTO ? moment(result[0].DTINATIVAUTO, 'DD/MM/YYYY').format(this.utilities.getFormatLocale()) : '';
          rowDadosRecipe.DSPRATO = result[0].DSPRATO;
          rowDadosRecipe.QTMEDICASEPRATPADR = result[0].QTMEDICASEPRATPADR;
          
          if (result[0].CDPRODPRODUCAO) {
            const prodPA = {
              NMPRODUTO: result[0].NMPRODUTO,
              CDPRODUTO: result[0].CDPRODPRODUCAO,
              CDARVPROD: result[0].CDARVPROD,
            };
            prodAcabado.datasource.data = [prodPA];
            prodAcabado.setValue(result[0].CDARVPROD);
          } else rowRefeicoesTransp.CDPRODPRODUCAO = '';

          rowRefeicoesTransp.IDFINALPADR = result[0].IDFINALPADR;
          rowRefeicoesTransp.IDPRODCHILL = result[0].IDPRODCHILL;
          rowRefeicoesTransp.QTLIMSEGPADR = result[0].QTLIMSEGPADR;
          rowRefeicoesTransp.IDTPPRODCHILL = result[0].IDTPPRODCHILL;
          rowDadosTransporte.QTCONTAIPRAT = result[0].QTCONTAIPRAT;
          rowDadosTransporte.VRPEMINPADR = result[0].VRPEMINPADR;
          rowDadosTransporte.QTAMOSTRPADR = result[0].QTAMOSTRPADR;
        } else {
          this.isNew = true;
          rowDadosRecipe.IDATIVO = 'S';
          rowDadosRecipe.QTMEDICASEPRATPADR = '0';
          rowDadosRecipe.NRCORPRIPRAT = '';
          rowRefeicoesTransp.IDFINALPADR = 'M';
          rowDadosTransporte.QTCONTAIPRAT = '0';
          rowDadosTransporte.VRPEMINPADR = '0';
          rowDadosTransporte.QTAMOSTRPADR = '0';
        }
      });
    } else if (tab === 'subTabReceitaLocal') {
      const form = Metadata.getInstance<Form>('formDadosReceitaLocal');
      const formLocal = Metadata.getInstance<Form>('formDadosReceitaLocal').value;
      const refTranspLocal = Metadata.getInstance<Form>('formRefeicoesTranspLocal').value;
      const tranpLocal = Metadata.getInstance<Form>('formDadosTransporteLocal').value;
      const medCase = form.getChildInstance<Select>('CDMEDICASE');
      const prodAcabado = Metadata.getInstance<Form>('formRefeicoesTranspLocal').getChildInstance<Select>('CDPRODPRODUCAO');
      
      const formFiltroLocal = Metadata.getInstance<Form>('FORM_LOCAL');
      const cdfilial = formFiltroLocal.value.BRANCH_LOCAL.CDFILIAL;
      
      const filter: IDictionary = [];
      this.utilities.onMountedFilterFind(filter, 'CDPRATO', '=', plate);
      this.utilities.onMountedFilterFind(filter, 'CDFILAUXPRAT', '=', filAux);
      this.utilities.onMountedFilterFind(filter, 'CDFILIAL', '=', cdfilial);
      Http.post('/find/pratFiliProdAcabado', { requestType: 'FilterData', filter }).then((callback) => {
        const result = callback.data.data;
        if (result.length > 0) {
          this.isNew = false;
          const formatHour = result[0].HRVALCONFILI ? this.formatHour(result[0].HRVALCONFILI) : null;
          if (result[0].CDMEDICASE) {
            const medidaCaseira = {
              NMMEDICASE: result[0].NMMEDICASE,
              CDMEDICASE: result[0].CDMEDICASE,
            };
            medCase.datasource.data = [medidaCaseira];
            medCase.setValue(result[0].CDMEDICASE);
          } else formLocal.CDMEDICASE = '';
          
          formLocal.NMFANTASIAPF = result[0].NMFANTASIAPF;
          formLocal.NRCORPRIFILI = this.utilities.TColorToHex(result[0].NRCORPRIFILI);
          formLocal.HRVALCONFILI = formatHour;
          formLocal.QTPADRAOFILI = result[0].QTPADRAOFILI.toFixed(5);
          formLocal.IDATIVO = result[0].IDATIVO;
          formLocal.DTINATIVAUTO = result[0].DTINATIVAUTO ? moment(result[0].DTINATIVAUTO, 'DD/MM/YYYY').format(this.utilities.getFormatLocale()) : '';
          formLocal.DSPRATO = result[0].DSPRATO;
          formLocal.QTMEDICASEPRATFILI = result[0].QTMEDICASEPRATFILI;
          
          if (result[0].CDPRODPRODUCAO) {
            const prodPA = {
              NMPRODUTO: result[0].NMPRODUTO,
              CDPRODUTO: result[0].CDPRODPRODUCAO,
              CDARVPROD: result[0].CDARVPROD,
            };
            prodAcabado.datasource.data = [prodPA];
            prodAcabado.setValue(result[0].CDARVPROD);
          } else refTranspLocal.CDPRODPRODUCAO = '';
          
          refTranspLocal.IDFINALFILI = result[0].IDFINALFILI;
          refTranspLocal.IDPRODCHILL = result[0].IDPRODCHILL;
          refTranspLocal.QTLIMSEGFILI = result[0].QTLIMSEGFILI;
          refTranspLocal.IDTPPRODCHILL = result[0].IDTPPRODCHILL;
          tranpLocal.QTCONTAIFILI = result[0].QTCONTAIFILI;
          tranpLocal.VRPEMINFILI = result[0].VRPEMINFILI;
          tranpLocal.QTAMOSTRFILI = result[0].QTAMOSTRFILI;
        } else {
          this.isNew = true;
          formLocal.IDATIVO = 'S';
          formLocal.QTMEDICASEPRATFILI = '0';
          formLocal.NRCORPRIFILI = '';
          refTranspLocal.IDFINALFILI = 'M';
          tranpLocal.QTCONTAIFILI = '0';
          tranpLocal.VRPEMINFILI = '0';
          tranpLocal.QTAMOSTRFILI = '0';
        }
      });
    } else if (tab === 'subTabReceitaServico') {
      const form = Metadata.getInstance<Form>('formDadosReceitaServico');
      const formServico = Metadata.getInstance<Form>('formDadosReceitaServico').value;
      const refTranspServico = Metadata.getInstance<Form>('formRefeicoesTranspServico').value;
      const tranpServico = Metadata.getInstance<Form>('formDadosTransporteServico').value;
      const medCase = form.getChildInstance<Select>('CDMEDICASE');
      const prodAcabado = Metadata.getInstance<Form>('formRefeicoesTranspServico').getChildInstance<Select>('CDPRODPRODUCAO');
      
      const formFiltroServico = Metadata.getInstance<Form>('FORM_SERVICE');
      const cdfilial = formFiltroServico.value.BRANCH_SERVICE.CDFILIAL;
      const cdserv = formFiltroServico.value.SERVICE.CDSERVFILI;
      
      const filter: IDictionary = [];
      this.utilities.onMountedFilterFind(filter, 'CDPRATO', '=', plate);
      this.utilities.onMountedFilterFind(filter, 'CDFILAUXPRAT', '=', filAux);
      this.utilities.onMountedFilterFind(filter, 'CDFILIAL', '=', cdfilial);
      this.utilities.onMountedFilterFind(filter, 'CDSERVFILI', '=', cdserv);
      Http.post('/find/pratServProdAcabado', { requestType: 'FilterData', filter }).then((callback) => {
        const result = callback.data.data;
        if (result.length > 0) {
          this.isNew = false;
          const formatHour = result[0].HRVALCONSERV ? this.formatHour(result[0].HRVALCONSERV) : null;
          if (result[0].CDMEDICASE) {
            const medidaCaseira = {
              NMMEDICASE: result[0].NMMEDICASE,
              CDMEDICASE: result[0].CDMEDICASE,
            };
            medCase.datasource.data = [medidaCaseira];
            medCase.setValue(result[0].CDMEDICASE);
          } else formServico.CDMEDICASE = '';
          
          formServico.NMFANTASIASE = result[0].NMFANTASIASE;
          formServico.NRCORPRISERV = this.utilities.TColorToHex(result[0].NRCORPRISERV);
          formServico.HRVALCONSERV = formatHour;
          formServico.QTPADRAOSERV = result[0].QTPADRAOSERV.toFixed(5);
          formServico.IDATIVO = result[0].IDATIVO;
          formServico.DTINATIVAUTO = result[0].DTINATIVAUTO ? moment(result[0].DTINATIVAUTO, 'DD/MM/YYYY').format(this.utilities.getFormatLocale()) : '';
          formServico.DSPRATO = result[0].DSPRATO;
          formServico.QTMEDICASEPRATSERV = result[0].QTMEDICASEPRATSERV;
          
          if (result[0].CDPRODPRODUCAO) {
            const prodPA = {
              NMPRODUTO: result[0].NMPRODUTO,
              CDPRODUTO: result[0].CDPRODPRODUCAO,
              CDARVPROD: result[0].CDARVPROD,
            };
            prodAcabado.datasource.data = [prodPA];
            prodAcabado.setValue(result[0].CDARVPROD);
          } else refTranspServico.CDPRODPRODUCAO = '';
          
          refTranspServico.IDFINALSERV = result[0].IDFINALSERV;
          refTranspServico.IDPRODCHILL = result[0].IDPRODCHILL;
          refTranspServico.QTLIMSEGSERV = result[0].QTLIMSEGSERV;
          refTranspServico.IDTPPRODCHILL = result[0].IDTPPRODCHILL;
          tranpServico.QTCONTAISERV = result[0].QTCONTAISERV;
          tranpServico.VRPEMINSERV = result[0].VRPEMINSERV;
          tranpServico.QTAMOSTRSERV = result[0].QTAMOSTRSERV;
        } else {
          this.isNew = true;
          formServico.IDATIVO = 'S';
          formServico.QTMEDICASEPRATSERV = '0';
          formServico.NRCORPRISERV = '';
          refTranspServico.QTCONTAISERV = 'M';
          tranpServico.QTCONTAISERV = '0';
          tranpServico.VRPEMINSERV = '0';
          tranpServico.QTAMOSTRSERV = '0';
        }
      });
    } else if (tab === 'subTabReceitaTipoCard') {
      const formTipoCard = Metadata.getInstance<Form>('formDadosReceitaTipoCard').value;
      const tranpTipoCard = Metadata.getInstance<Form>('formDadosTransporteTipoCard').value;
      
      const formFiltroTpCard = Metadata.getInstance<Form>('FORM_TIPOCARD');
      const tipocard = formFiltroTpCard.value.TIPOCARD?.CDTIPOCARD;
      if (tipocard === undefined) {
        this.utilities.showAlert('alert', 'pme::tipoCardapioDeveSerInformado', 'error');
        return;
      }
      Http.post('/prattpcard', { plate, filAux, tipocard }).then((callback) => {
        const result = callback.data.dataset.columnsToReturn;
        if (result.length > 0) {
          this.isNew = false;
          const formatHour = result[0].HRVALCONPRATTP ? this.formatHour(result[0].HRVALCONPRATTP) : null;
          formTipoCard.NMFANTASIAPRTP = result[0].NMFANTASIAPRTP;
          formTipoCard.NRCORPRIPRATTP = this.utilities.TColorToHex(result[0].NRCORPRIPRATTP);
          formTipoCard.HRVALCONPRATTP = formatHour;
          formTipoCard.QTPADRAOPRATTP = result[0].QTPADRAOPRATTP.toFixed(5);
          formTipoCard.IDATIVO = result[0].IDATIVO;
          formTipoCard.DTINATIVAUTO = result[0].DTINATIVAUTO ? moment(result[0].DTINATIVAUTO, 'DD/MM/YYYY').format(this.utilities.getFormatLocale()) : '';
          formTipoCard.DSPRATO = result[0].DSPRATO;
          tranpTipoCard.QTCONTAIPRATTP = result[0].QTCONTAIPRATTP;
          tranpTipoCard.VRPEMINPADRTP = result[0].VRPEMINPADRTP;
          tranpTipoCard.QTAMOSTRPADRTP = result[0].QTAMOSTRPADRTP;
        } else {
          this.isNew = true;
          formTipoCard.IDATIVO = 'S';
          formTipoCard.QTMEDICASEPRATPADR = '0';
          formTipoCard.NRCORPRIPRATTP = '';
          tranpTipoCard.QTCONTAIPRATTP = '0';
          tranpTipoCard.VRPEMINPADRTP = '0';
          tranpTipoCard.QTAMOSTRPADRTP = '0';
        }
      });
    }
    
    if (this.qtdLiqBrut === '') this.getParamPlaneja();
  }
  
  public formatHour(hour: any) {
    const hourString = hour.toString();
    let valueHour = Math.trunc(hour).toString();
    const valueMinutes = hourString.substr(-1);
    let minutos;
    if (valueMinutes.length === 1) minutos = (valueMinutes / 10);
    else minutos = valueMinutes;
    const totalMin = minutos * 60;
    let totalMinString = totalMin.toString();
    if (totalMinString.length === 1) totalMinString = `0${totalMin}`;
    if (valueHour.length === 1) valueHour = `0${valueHour}`;
    const total = `${valueHour}:${totalMinString}`;
    return total;
  }
  
  public requiredQtd({ component }: IEventParam<Select>) {
    const tab = this.setNameTab;
    let form;
    let qtd: any;
    if (tab === 'subTabReceitaPadrao') {
      form = Metadata.getInstance<Form>('formDadosReceita');
      qtd = form?.getChildInstance('QTMEDICASEPRATPADR');
    } else if (tab === 'subTabReceitaLocal') {
      form = Metadata.getInstance<Form>('formDadosReceitaLocal');
      qtd = form?.getChildInstance('QTMEDICASEPRATFILI');
    } else if (tab === 'subTabReceitaServico') {
      form = Metadata.getInstance<Form>('formDadosReceitaServico');
      qtd = form?.getChildInstance('QTMEDICASEPRATSERV');
    } else if (tab === 'subTabReceitaTipoCard') {
      form = Metadata.getInstance<Form>('formDadosReceitaTipoCard');
      qtd = form?.getChildInstance('QTMEDICASEPRATPADR');
    }
    
    if (component.value === undefined || component.value === null || component.value === '') {
      qtd.removeValidation('required');
      qtd.value = null;
      qtd.disabled = true;
    } else {
      qtd.disabled = false;
    }
  }
  
  public disableButtonsRecipeTrue() {
    const tab = this.setNameTab;
    let cancel;
    let save;
    if (tab === 'subTabReceitaPadrao') {
      cancel = Metadata.getInstance<Button>('cancelButtonRecipePadrao');
      save = Metadata.getInstance<Button>('saveButtonRecipePadrao');
    } else if (tab === 'subTabReceitaLocal') {
      cancel = Metadata.getInstance<Button>('cancelButtonRecipeLocal');
      save = Metadata.getInstance<Button>('saveButtonRecipeLocal');
    } else if (tab === 'subTabReceitaServico') {
      cancel = Metadata.getInstance<Button>('cancelButtonRecipeServico');
      save = Metadata.getInstance<Button>('saveButtonRecipeServico');
    } else if (tab === 'subTabReceitaTipoCard') {
      cancel = Metadata.getInstance<Button>('cancelButtonRecipeTpCard');
      save = Metadata.getInstance<Button>('saveButtonRecipeTpCard');
    }
    
    if (cancel) cancel.disabled = true;
    if (save) save.disabled = true;
    this.setValuesRecipe();
  }
  
  public disableButtonsRecipeFalse() {
    const tab = this.setNameTab;
    let cancel;
    let save;
    if (tab === 'subTabReceitaPadrao') {
      cancel = Metadata.getInstance<Button>('cancelButtonRecipePadrao');
      save = Metadata.getInstance<Button>('saveButtonRecipePadrao');
    } else if (tab === 'subTabReceitaLocal') {
      cancel = Metadata.getInstance<Button>('cancelButtonRecipeLocal');
      save = Metadata.getInstance<Button>('saveButtonRecipeLocal');
    } else if (tab === 'subTabReceitaServico') {
      cancel = Metadata.getInstance<Button>('cancelButtonRecipeServico');
      save = Metadata.getInstance<Button>('saveButtonRecipeServico');
    } else if (tab === 'subTabReceitaTipoCard') {
      cancel = Metadata.getInstance<Button>('cancelButtonRecipeTpCard');
      save = Metadata.getInstance<Button>('saveButtonRecipeTpCard');
    }
    
    if (cancel) cancel.disabled = false;
    if (save) save.disabled = false;
  }
  
  public validDateInactivation() {
    const tab = this.setNameTab;
    let form;
    if (tab === 'subTabReceitaPadrao') form = Metadata.getInstance<Form>('formDadosReceita').value;
    else if (tab === 'subTabReceitaLocal') form = Metadata.getInstance<Form>('formDadosReceitaLocal').value;
    else if (tab === 'subTabReceitaServico') form = Metadata.getInstance<Form>('formDadosReceitaServico').value;
    else if (tab === 'subTabReceitaTipoCard') form = Metadata.getInstance<Form>('formDadosReceitaTipoCard').value;
    const date = form?.DTINATIVAUTO;
    
    if (date) {
      if (moment(date, 'DD/MM/YYYY').isAfter(moment())) return true;
      this.utilities.showAlert('alert', 'pme::dataInativacaoMaiorDataAtual', 'error');
      return false;
    }
    return true;
  }
}
