import {
  Form, Grid, Column, TreeGridEditable, Tabs, Date as ZdDate, Select,
} from '@zeedhi/common';
import {
  Metadata, IDictionary, Loader, I18n, Http, dayjs as moment, IEventParam,
} from '@zeedhi/core';

export class Pla01304ConsultaCustosController {
  private utilities = Loader.getInstance('Utilities');
  
  public appController = Loader.getInstance('AppController');
  
  public reportController = Loader.getInstance('ReportController');
  
  public pratoController = Loader.getInstance('Pla01300PratoController');
  
  public currency: any = '';
  
  public dateProcess: any = '';
  
  public setValueTipoCusto({ component }: IEventParam<Select>) {
    this.reportController.defaultTpCusto(component);
  }
  
  public async setValueDate({ component }: IEventParam<ZdDate>) {
    const formUnidade = Metadata.getInstance<Form>('formUnidade').value;
    const cdfilial = formUnidade.UNIDADEFORM.CDFILIAL;
    this.utilities.getProcessDate(cdfilial).then((date: string) => {
      component.value = date;
      this.filterPlatesGrid();
    });
  }
  
  public getParamPlanejaEFilial() {
    this.currencySymbol();
  }
  
  private async filterPlatesGrid() {
    const filter: IDictionary = [];
    const tab = Metadata.getInstance<Tabs>('tabPratos');
    if (tab) {
      let container: any = '';
      let form: any = '';
      let formCustos: any = '';
      let grid: any = '';
      let tipo: any = '';
      let date: any = '';
      let totalCustos: any = '';
      let safraResult: any = '';
    
      const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
      const currentRowTree = plateTree.datasource.currentRow;
      const formUnidade = Metadata.getInstance<Form>('formUnidade').value;
      
      let cdFiliCoast: any;
      let cdServFili: any;
      let cdTipoCard: any;
      let tipoRec: string;
      
      if (tab.activeTab === 3) {
        container = Metadata.getInstance('consulta_custos-TipoCard');
        form = container.getChildInstance('FORMCONSULTACUSTOS-TipoCard');
        grid = form.getChildInstance('CONSULTACUSTOS-TipoCard');
        formCustos = form.getChildInstance('formCustos-TipoCard');
        tipo = form.getChildInstance('NMTIPOCUSTO');
        date = form.getChildInstance('DATEINPUT');
        totalCustos = formCustos.getChildInstance('CustoPerCaptaPratoResult');
        safraResult = formCustos.getChildInstance('SafraResult');
        const formTpCard = Metadata.getInstance<Form>('FORM_TIPOCARD').value;
        cdTipoCard = formTpCard.TIPOCARD?.CDTIPOCARD;
        if (cdTipoCard === undefined) {
          grid.datasource.data = [];
          this.utilities.showAlert('alert', 'pme::tipoCardapioDeveSerInformado', 'error');
          return;
        }
        tipoRec = 'TC';
      } else if (tab.activeTab === 4) {
        container = Metadata.getInstance('consulta_custos-Local');
        form = container.getChildInstance('FORMCONSULTACUSTOS-Local');
        formCustos = form.getChildInstance('formCustos-Local');
        grid = form.getChildInstance('CONSULTACUSTOS-Local');
        tipo = form.getChildInstance('NMTIPOCUSTO');
        date = form.getChildInstance('DATEINPUT');
        totalCustos = formCustos.getChildInstance('CustoPerCaptaPratoResult');
        safraResult = formCustos.getChildInstance('SafraResult');
        const formLocal = Metadata.getInstance<Form>('FORM_LOCAL').value;
        cdFiliCoast = formLocal.BRANCH_LOCAL.CDFILIAL;
        tipoRec = 'F';
      } else if (tab.activeTab === 5) {
        container = Metadata.getInstance('consulta_custos-Servico');
        form = container.getChildInstance('FORMCONSULTACUSTOS-Servico');
        formCustos = form.getChildInstance('formCustos-Servico');
        grid = form.getChildInstance('CONSULTACUSTOS-Servico');
        tipo = form.getChildInstance('NMTIPOCUSTO');
        date = form.getChildInstance('DATEINPUT');
        totalCustos = formCustos.getChildInstance('CustoPerCaptaPratoResult');
        safraResult = formCustos.getChildInstance('SafraResult');
        
        const formService = Metadata.getInstance<Form>('FORM_SERVICE').value;
        cdFiliCoast = formService.BRANCH_SERVICE.CDFILIAL;
        cdServFili = formService.SERVICE.CDSERVFILI;
        tipoRec = 'S';
      } else if (tab.activeTab === 2) {
        container = Metadata.getInstance('consulta_custos-Padrao');
        form = container.getChildInstance('FORMCONSULTACUSTOS-Padrao');
        formCustos = form.getChildInstance('formCustos-Padrao');
        grid = form.getChildInstance('CONSULTACUSTOS-Padrao');
        tipo = form.getChildInstance('NMTIPOCUSTO');
        date = form.getChildInstance('DATEINPUT');
        totalCustos = formCustos.getChildInstance('CustoPerCaptaPratoResult');
        safraResult = formCustos.getChildInstance('SafraResult');
        tipoRec = 'P';
      }
      
      cdFiliCoast = cdFiliCoast === undefined ? null : cdFiliCoast;
      cdServFili = cdServFili === undefined ? null : cdServFili;
      cdTipoCard = cdTipoCard === undefined ? null : cdTipoCard;
  
      let custoTotalPerCapta: any = 0;
      let percentualSafra = 0;
  
      setTimeout(() => {
        if (date.value !== null && tipo.value !== null) {
          this.utilities.onMountedFilterFind(filter, 'CDFILIALCUSTO', '=', formUnidade.UNIDADEFORM.CDFILIAL);
          this.utilities.onMountedFilterFind(filter, 'NRTIPOCUSTO', '=', tipo.value);
          this.utilities.onMountedFilterFind(filter, 'DTCUSTO', '=', date.value);
          this.utilities.onMountedFilterFind(filter, 'CDSERVFILI', '=', cdServFili);
          this.utilities.onMountedFilterFind(filter, 'MES', '=', moment(date.value, 'DD/MM/YYYY').format('MM'));
          this.utilities.onMountedFilterFind(filter, 'CDFILAUXPRAT', '=', currentRowTree.CDFILAUXPRAT);
          this.utilities.onMountedFilterFind(filter, 'CDPRATO', '=', currentRowTree.CDPRATO);
          this.utilities.onMountedFilterFind(filter, 'CDFILIAL', '=', cdFiliCoast);
          this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', this.appController.getCdCentral());
          this.utilities.onMountedFilterFind(filter, 'CDTIPOCARD', '=', cdTipoCard);
          this.utilities.onMountedFilterFind(filter, 'TIPO', '=', tipoRec);
          
          grid.datasource.filter = filter;
          grid.datasource.get().then(() => {
            grid.datasource.data.forEach((row: any) => {
              custoTotalPerCapta += row.CUSTONOPRATO;
              if (row.IDINTESAFR != null) {
                if (row.IDINTESAFR === '2') {
                  if ((custoTotalPerCapta * 100) / 2 !== 0) percentualSafra += (row.CUSTONOPRATO / (custoTotalPerCapta * 100) / 2);
                }
                if (row.IDINTESAFR === '3') {
                  if (custoTotalPerCapta * 100 !== 0) percentualSafra += row.CUSTONOPRATO / (custoTotalPerCapta * 100);
                }
              } else row.IDINTESAFR = 0;
            });
            this.setCurrencyRow(grid);
            totalCustos.text = this.utilities.formatValuesToFixed(custoTotalPerCapta, 5);
            safraResult.text = `${this.utilities.formatValuesToFixed(percentualSafra, 2)}${' %'}`;
            if (percentualSafra > 0) {
              Http.post('/prato/returnPercentSafra', { requestType: 'Row', percentualSafra }).then((returnSafra) => {
                const returnColorSafra = returnSafra.data.dataset.result[0].NRCORSINSAFR;
                const colorSafra = this.utilities.TColorToHex(returnColorSafra.toString());
                document.getElementsByName('SafraResult')[0].style.color = colorSafra;
              });
            }
          });
        } else {
          grid.datasource.data = [];
          totalCustos.text = this.utilities.formatValuesToFixed(0, 5);
          safraResult.text = `${this.utilities.formatValuesToFixed(0, 2)}${' %'}`;
        }
      }, 2);
    }
  }
  
  public returnColumn(component: Grid, nmColuna: string) {
    return component.columns.find((column) => column.name === nmColuna);
  }
  
  public setCurrencyRow(grid: Grid) {
    const custo = this.returnColumn(grid, 'CUSTO') as Column;
    const custoPrato = this.returnColumn(grid, 'CUSTONOPRATO') as Column;
    const currency = this.currency;
    
    custo.componentProps.mask.currencySymbol = currency;
    custoPrato.componentProps.mask.currencySymbol = currency;
  }
  
  public currencySymbol() {
    const currency = localStorage.getItem('CURRENCY');
    this.currency = currency;
  }
  
  private checkLanguage() {
    const language = this.utilities.getLanguage() as string;
    let idioma = null;
    if (language != null && language.includes('en')) idioma = 'I';
    else if (language != null && language.includes('es')) idioma = 'E';
    return idioma;
  }
  
  private getSafColor({ row }: IDictionary) {
    let decColor = 0;
    const nrcorsafra = this.utilities.hexToRgbInline(this.pratoController.nrcorsafra).split('.', 3);
    const nrcorinicsaf = this.utilities.hexToRgbInline(this.pratoController.nrcorinicsaf).split('.', 3);
    const nrcorentrsaf = this.utilities.hexToRgbInline(this.pratoController.nrcorentrsaf).split('.', 3);
    
    if (row.IDINTESAFR > 0) {
      if (row.IDINTESAFR === '1') {
        decColor = this.utilities.RGBToTColor(nrcorsafra[0], nrcorsafra[1], nrcorsafra[2]);
        row.SAFRA = I18n.translate('pme::safra');
      }
      if (row.IDINTESAFR === '2') {
        decColor = this.utilities.RGBToTColor(nrcorinicsaf[0], nrcorinicsaf[1], nrcorinicsaf[2]);
        row.SAFRA = I18n.translate('pme::inicioFimSafra');
      }
      if (row.IDINTESAFR === '3') {
        decColor = this.utilities.RGBToTColor(nrcorentrsaf[0], nrcorentrsaf[1], nrcorentrsaf[2]);
        row.SAFRA = I18n.translate('pme::entreSafra');
      }
    } else row.IDINTESAFR = '';
    
    const color = this.utilities.TColorToHex(decColor.toString());
    
    return color;
  }
}
